/*
 * Created on 11.12.2004
 *
 */
package sibiriaGame;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;



/**
 * This class implements a simple graphical user interface with a text entry
 * area, a text output area and an optional image.
 * 
 * @author Michael Kolling
 * @version 1.0 (Jan 2003)
 */
public class UserInterface extends JFrame
{
    private static GameEngine engine;
  
    private JPanel mainPanelright;
	private JPanel mainPanelleft;
    private JPanel actionButtonPanel;	
	private JPanel itemButtonPanel;		
	private JTextArea userInform;
	private JLabel inputtextfield;
	private JLabel image;
	private JLabel timetext;
	private JLabel moneytext;
	private JLabel freezetext;
	private JLabel hungertext;
	private JLabel thirsttext;
	private JLabel phototext;
	private DialogWindow window;

	//private JButton takePhoto;

	private JPanel looknice;

	
	
	
    /**
     * Construct a UserInterface. As a parameter, a Game Engine
     * (an object processing and executing the game commands) is
     * needed.
     * 
     * @param gameEngine  The gameEngine object implementing the game logic.
     */
    public UserInterface(GameEngine gameEngine) 
    
    {        
        super("Travelling to Sibiria");	//inherited constructor with argument: title of window
        engine = gameEngine;
		buildInterface();//main method of the user interface
		pack();
		//set visible explicitly from game engine
		window = new DialogWindow(engine);//a dialogWindow is created
		window.setLocationRelativeTo(image);//the dialog will be displayed with the left upper corner in the middle of the image that represents the room
		setDefaultCloseOperation(EXIT_ON_CLOSE);
    }
    
    /**
	 * makes the dialogWindow visible
	 */
	public void dialogWindow() {		
		window.setVisible(true);	
	}


    
	/**
	 * sets up the containers and the invokes the construction of the different components of the user interface	 * 
	 */
	private void buildInterface() {
		//the different containers
		mainPanelright = new JPanel();
		mainPanelleft = new JPanel();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(mainPanelleft, BorderLayout.WEST);
		getContentPane().add(mainPanelright, BorderLayout.EAST);
		mainPanelleft.setLayout(new BorderLayout());
		mainPanelright.setLayout(new BorderLayout());
		
		//the different fields in the user interface
		buildImage();	//displays the image for representing the actual room	
		buildActionButtons();//displays the buttons for the most common actions - go help quit wait talk_to_person shout_and_wave
		buildOutputTextField();//here information about the actual state of the game world, help information etc is displayed.
		buildDisplayInfo();//displays following info: money hunger thirst freeze time #of photos
		buildItemButtons();//shows the actual available items 1)in room 2) carried 3) worn 
		buildTextInputField();//allows the user to insert free text
		TakePhoto();
	}



	/**
	 * set up the field in which the user can input the commands
	 */
	private void buildTextInputField() {
	
			final JTextField textfield = new JTextField(50);
			JButton doso = new JButton("do so");//this button allows to send the input to the game engine
			JPanel textIn = new JPanel();
			mainPanelleft.add(textIn, BorderLayout.SOUTH);//let this be at the bottom of the left container
			textIn.add(textfield);//standard layout, one textfield next to button
			textIn.add(doso);
			textIn.getRootPane().setDefaultButton(doso);//then the player can just press ENTER and does not have to click the button every time
			
			doso.addActionListener(new ActionListener(){//add an action listener for the button

				public void actionPerformed(ActionEvent arg0) {
					if (! textfield.getText().equals("")){//if there is some input
					engine.interpretCommand(textfield.getText());//get it and forward it to the gameengine interpredCommand method
					textfield.setText("");//clear the textfield after that					
					}
				}
			});			
	}

	/**
	 * builds a list of the buttons for the items that are in the room, carried by the player of worn
	 *
	 */
	private void buildItemButtons() {
		
		itemButtonPanel = new JPanel();		//in this list the buttons and three labels that show where each item is are put 
		itemButtonPanel.setLayout(new GridLayout(0,1));//one under the other
		JPanel itemButtonPanelContainer = new JPanel();//an additional container  so that it does not take all the space
		itemButtonPanelContainer.setLayout(new FlowLayout());
		itemButtonPanelContainer.add(itemButtonPanel);
		updateItemButtons();//a method that determines which buttons are actually needed and builds them
		mainPanelright.add(itemButtonPanelContainer,BorderLayout.CENTER);//the list of buttons will be on the top in the right container
		
	}
	
	/**
	 * 
	 *
	 */
	private void TakePhoto(){
		JButton takePhoto = new JButton();	
		looknice = new JPanel();
		//looknice.setLayout(new BorderLayout());
		looknice.add(takePhoto);//, BorderLayout.CENTER);
		ImageIcon camera = new ImageIcon("camera.JPG");//get from the gameEngine the information of the image location
		takePhoto.setIcon(camera);//put the image to the panel
		mainPanelright.add(looknice,BorderLayout.SOUTH);
		
		takePhoto.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
//				give to the gameEngine some recognition word (apply) and the informtation where the item is ("item_in_room") and the description of the item that appeared on the button			
				engine.interpretCommand("take photo");  
				
			}
		});   
	}
	
	
	/**
	 * which buttons are actually needed and builds them
	 * as the itemlist is logically made of three subparts but these should not take a different amount of space.
	 * 
	 */
	private void updateItemButtons() {
		itemButtonPanel.removeAll(); //clear the old list to get new information	
		HashMap actualItems = engine.getItems();//get the list of items in room
		Set itemkeys = actualItems.keySet();		
		JLabel roomitemlist = new JLabel("items in room: ");//create an explanationlabel
		itemButtonPanel.add(roomitemlist);		//add it in the first place
        for(Iterator iter = itemkeys.iterator(); iter.hasNext(); )//iterate over all itemdescriptions of items in room
        	{
        	JButton button = new JButton((String) iter.next());
        	button.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
//					give to the gameEngine some recognition word (apply) and the informtation where the item is ("item_in_room") and the description of the item that appeared on the button			
					engine.interpretCommand("apply item_in_room " + e.getActionCommand());  
				}
			});   
        	itemButtonPanel.add(button);
        	}
		
        
        HashMap actualWorn = engine.getPlayer().getWorn();	//same procedure for worn items
        Set wkeys = actualWorn.keySet();
		JLabel wornitemslist = new JLabel("worn things: ");
		itemButtonPanel.add(wornitemslist);		
		for(Iterator iter = wkeys.iterator(); iter.hasNext(); )
        	{
        	JButton button = new JButton((String) iter.next());
        	itemButtonPanel.add(button);
        	button.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					engine.interpretCommand("apply worn "+e.getActionCommand());  	//only difference to the item_in_room one is the information "worn"			
				}
			});  
        	}
		
        
		HashMap actualCarried = engine.getPlayer().getCarried();//same procedure for carried items
		Set ckeys = actualCarried.keySet();
		JLabel carrieditemslist = new JLabel("carried things: ");
    	itemButtonPanel.add(carrieditemslist);
        for(Iterator iter = ckeys.iterator(); iter.hasNext(); )
        	{
        	JButton button = new JButton((String) iter.next());
        	itemButtonPanel.add(button);
        	button.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					engine.interpretCommand("apply carried "+e.getActionCommand());  				
				}
			});  
        	}   
	}

	/**
	 * There is some information about the player and his environment shown.(hunger, thirst, money etc.) 
	 * This method constructs the fields for this information
	 */
	private void buildDisplayInfo() {
		freezetext = new JLabel();//build labels for each of the information elements
		hungertext = new JLabel();
		thirsttext = new JLabel();
		moneytext = new JLabel();
		timetext = new JLabel();
		phototext = new JLabel();
		updateInfo();//gets the information needed from the gameengine and sets text to the labels
		JPanel situationPanel = new JPanel();
		situationPanel.setBorder(new EmptyBorder(10,10,10,10));//to space it a bit away from the other components
		situationPanel.setLayout(new GridLayout(0,1));//all the labels are arranged vertically 
		situationPanel.add(moneytext);
		situationPanel.add(timetext);
		situationPanel.add(hungertext);
		situationPanel.add(thirsttext);
		situationPanel.add(freezetext);
		situationPanel.add(phototext);
		mainPanelright.add(situationPanel,BorderLayout.NORTH);//the information will be at the right on top of the item buttons
	}

	/**
	 * this method puts the information texts to the labels created in buildDisplayInfo()
	 * and for this gets the relevant information from the gameEngine
	 * thus when it is called the information for the user is updated.
	 * 
	 */
	//TODO:information directly from engine so that interface need less information of where to get information from
	public void updateInfo() {
		freezetext.setText("freezing level: "+ engine.getPlayer().getFreeze() + " (max: "+ engine.getMaxFreeze()+")");
		hungertext.setText("hunger: "+ engine.getPlayer().getHunger()+ " (best not above 0)");
		thirsttext.setText("thirst: "+ engine.getPlayer().getThirst()+ " (max: "+ engine.getMaxThirst()+")");
		moneytext.setText("money: "+ engine.getPlayer().getMoney() + engine.getPlayer().getCurrency());
		timetext.setText("time: "+ engine.getTime() + " o'clock ");
		phototext.setText("photos: "+ engine.getPhotos()+ " (max: "+ engine.getMaxPhotos()+")");
		pack();
	}
    
	/**
	 * constructs the text area under the image from which the player is informed about the rooms,
	 * consequences of his commands and where help can be displayed.
	 */
	private void buildOutputTextField() {
		userInform = new JTextArea();//create textArea
		ScrollPane ScrollContainer = new ScrollPane();//create a scrollable container
		ScrollContainer.add(userInform);		//and put it into it
		mainPanelleft.add(ScrollContainer, BorderLayout.CENTER);//the outputtextfield is put to the left under the image representing the room
		userInform.setEditable(false);	//the user is not allowed to type text into this field.
		}



	/**
	 * adds new information for the user to the text output field
	 * @param commandLine
	 */
	public void println(String commandLine) {
		userInform.insert(commandLine + "\n\n",0);	//inserts new information to the text output by adding it to the top.
		//System.out.println(commandLine);
		pack();		
	}
	
	/**
	 * create a dialog that gives the player a choice of currently available exits
	 * @param a HashMap of the exits of a room
	 * @return a String that is the HashMap-Key of the exit(new room) chosen
	 */
	public String directionDialog(HashMap destinations) {
		destinations.put("stay",engine.getCurrent());//to allow the user to decide differently (not to go anywhere) this is important as time goes by when changing locations
		Set keys = destinations.keySet();//get keys(descriptions) of currently possible exits
		Object[] options = keys.toArray();//the button labels that will be represented to the user are stored in an array, which is required by the JOptionPane
		int ans = JOptionPane.showOptionDialog(null, "Where do you want to go?","go", 
				JOptionPane.DEFAULT_OPTION, JOptionPane.PLAIN_MESSAGE,
				null, options, options[0]);//gives a window with title "go", a label saying "where do you want to go" and the different options as buttons aligned horizontically under this label
		String answer = (String) options[ans];//answer stores the label of the button that was clicked
		return((String)options[ans]);		
	}
	
	
	/**
	 * builds the action buttons which then call the interpretCommand method of the game engine
	 */
	private void buildActionButtons() {
		
		actionButtonPanel = new JPanel();
		actionButtonPanel.setLayout(new GridLayout(1,0));//display the buttons in a line, take same space for every button but not whole space of window
		class simpleActionListener implements ActionListener{//used by all action buttons
			/* (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(ActionEvent e) {
				engine.interpretCommand(e.getActionCommand());//give the name of the button clicked to the interpretCommand method of the game engine				
			}			
		}
        //***************************************//same operations for all simple actions:
		JButton gobutton = new JButton("go");//argument is label that will be displayed on button
        gobutton.addActionListener(new simpleActionListener());   //add the action listener
        actionButtonPanel.add(gobutton);        	//add the button to the buttonPanel
        //**********************************************************
        JButton helpbutton = new JButton("help");     
        helpbutton.addActionListener(new simpleActionListener());
        actionButtonPanel.add(helpbutton);   
//      **********************************************************
        JButton quitbutton = new JButton("quit");
        quitbutton.addActionListener(new simpleActionListener());
        actionButtonPanel.add(quitbutton);        	
//      **********************************************************
        JButton talkbutton = new JButton("talk_to_person");
        talkbutton.addActionListener(new simpleActionListener());
        actionButtonPanel.add(talkbutton);  
//      ********************************************************** 
        JButton shoutbutton = new JButton("shout_and_wave");
        shoutbutton.addActionListener(new simpleActionListener());
        actionButtonPanel.add(shoutbutton);        	
//      **********************************************************
        JButton waitbutton = new JButton("wait");
        waitbutton.addActionListener(new simpleActionListener());
        actionButtonPanel.add(waitbutton);  
//      **********************************************************
        getContentPane().add(actionButtonPanel, BorderLayout.SOUTH);//the buttons will be displayed at the very bottom of the user interface		
	}




	/**
	 * builds the image that represents a room and places it in the top of the left panel
	 * 
	 */
	private void buildImage() {
		image = new JLabel();		
		ImageIcon icon = new ImageIcon((String) engine.getImage());//get from the gameEngine the information of the image location
		image.setIcon(icon);//put the image to the panel
		mainPanelleft.add(image,BorderLayout.NORTH);// put the image to the topleft of the window
		
	}
	
	/**
	 * updates the image representing for the representation of the actual room
	 */
	public void updateImage(){
		image.removeAll();//remove last icon
		ImageIcon icon = new ImageIcon((String) engine.getImage());//create new icon with picture location/name asked from game Engine
		image.setIcon(icon);//put the new image to the image label
		
	}

    
	/**
	 * updates the components displaying the image representing the room,
	 * the field that informs the player about his situation (hunger thirst money etc.)
	 * and about the items available / carried / worn	 
	 */
	public void update() {
		updateImage();
		updateInfo();
		updateItemButtons();
		
		if (!(engine.getPlayer().getCarried().containsKey("camera"))){
			looknice.setVisible(false);
		}
		else if (looknice.isVisible()== false){
			looknice.setVisible(true);
		}
		
		pack();		//some components might have changed size after update
	}




	/**
	 * creates a quit- dialogue that permits the player to confirm whether he wants to quit 
	 * and prevents him from accidently exiting the game.
	 * Displays the question Do you really want to quit and  under this two buttons 
	 * for the yes-no choice. Destroys the userInterface if yes is clicked.
	 * 
	 */
	public void close() {
		int s = JOptionPane.showConfirmDialog(null, 
				"Do you really want to quit?", "quit",  JOptionPane.YES_NO_OPTION);//title of window: quit, question asked:"Do you really want to quit?"
		if (s==0){  	//corresponds to "yes"	
			disposemain();//destroy user interface
			}
		
	}

	


	/**
	 * create a window that has a title displays an image and  a button to close it
	 * (used for take photo and won/lost end_of_game)
	 * @param string the title of the window
	 * @param string the location/name of the photo
	 * @param string the label that should be displayed on the closing button
	 */
	public void showPhoto(final String title,final String photo, final String buttonlabel) {
		class PhotoWindow extends JFrame {//every time the method is called a new instance of this class is created
			JPanel mainPhotoPanel;
			public PhotoWindow(){
				super(title);	//construct it and give title 
				buildPhoto();//main method for construction of this window
				setLocationRelativeTo(image);//the photowindow will be displayed with the left upper corner in the middle of the image that represents the room
				pack();
				setVisible(true);
			}

			/**
			 * display the image and a button that closed the window
			 */
			private void buildPhoto() {			
				getContentPane().setLayout(new BorderLayout());	
				buildLabel();//image will be displayed in here
				buildConfirmButton();	//button to close window
			}
			/**
			 * construct the image, with the overgiven second argument of the method that called this class
			 */
			private void buildLabel() {
				JLabel photolabel = new JLabel();//create a label for the image
				ImageIcon icon = new ImageIcon(photo);//photo is the second arguemnt of showPhoto(String,String)
				photolabel.setIcon(icon);//put the image to the label				
				getContentPane().add(photolabel,BorderLayout.CENTER);//display the image in the center of the window		
			}

			/**
			 * creates a button that then closes the window
			 */
			private void buildConfirmButton() {
				JButton cbutton = new JButton(buttonlabel);//create button and give it the label that will be displayed on it
				JPanel buttondesign = new JPanel();//additional container so that the button is not as long as the image wide
				buttondesign.add(cbutton);//add the button to this container
				getContentPane().add(buttondesign,BorderLayout.SOUTH);//and the container to the bottom of the window
				cbutton.addActionListener(new ActionListener(){

	    			public void actionPerformed(ActionEvent e) {
	    				dispose();//close the photowindow if button pressed
	    				if (engine.gameOver()){
	    					disposemain();
	    				}
	    			}
	    		});
			}
		}
		
		PhotoWindow window = new PhotoWindow();//every time the method is called a new instance of this class is created
		//window.alwaysOnTop();//toFront();
	}
	
	public void makeInvisible(){
			setVisible(false);
			//pack();
		
	}

	/**
	 * creates the StartWindow of the game, which contains a photo and a display text field 
	 * in which the game is introduced and the rules explained.
	 * The window also has a title("Travelling in Sibiria") and a button (start game) at the bottom
	 * 
	 * @param string the photo path
	 * @param String the introduction and rules
	 */
	public void startScreen(final String photo,final String introAndRules) {
		class StartWindow extends JFrame {
			JPanel mainPhotoPanel;//main panel
			public StartWindow(){
				super("start game \"Travelling in Sibiria\"");	//window created and title is set
				buildStart();//main method for this window, invokes construction of Image, text and button
				pack();
				setVisible(true);
			}

			/**
			 * set Layoutmanager for Window
			 *  invoke construction of Image, text and button
			 */
			private void buildStart() {			
				getContentPane().setLayout(new BorderLayout());	
				buildLabel();//constructs the field for the image and displays it
				buildInformtext();//constructs the text
				buildConfirmButton();	//constructs the button "start game"
			}
			/**
			 * constructs the image
			 */
			private void buildLabel() {
				JLabel photolabel = new JLabel();
				ImageIcon icon = new ImageIcon(photo);//image is found at location specified by first parameter of startScreen-method
				photolabel.setIcon(icon);//		put image to label	
				photolabel.setBorder(new EtchedBorder());//to have a nicer layout
				getContentPane().add(photolabel,BorderLayout.NORTH);	//image is on the top of the window			
			}
			/**
			 * construct the text field and fill it with text
			 */
			private void buildInformtext() {
				JTextArea rulesInform = new JTextArea();//create text area
				rulesInform.setEditable(false);	//should not be editable by player
				rulesInform.append(introAndRules);//insert the text
				rulesInform.setBorder(new EtchedBorder());//to have a nicer layout
				getContentPane().add(rulesInform,BorderLayout.CENTER);//display this component under the image		
				
				}
			/**
			 * build a button with label "start game" that gives to the game engine the information that
			 * the userInterface should be made visible 
			 */
			private void buildConfirmButton() {
				JButton cbutton = new JButton("start game");//create button with label"start game"
				JPanel buttondesign = new JPanel();//create container in which the bottom is put to prevent the bottom from being as wide as the text or image
				buttondesign.add(cbutton);//add the bottom to this container
				getContentPane().add(buttondesign,BorderLayout.SOUTH);//and the container to the window at the bottom
				cbutton.addActionListener(new ActionListener(){

	    			public void actionPerformed(ActionEvent e) {
	    				engine.setVisible();//gives to the game engine the information that the userInterface can now be set visible
	    				dispose();//disposes the start window
	    				
	    			}
	    		});
			}
		}
		StartWindow window = new StartWindow();//when the method is called, a new instance of the StartWindow class is created
	}
	
	/**
	 * destroys the userinterface
	 */
	public void disposemain(){
		//dispose();
		System.exit(EXIT_ON_CLOSE);
	}


	/**
	 * sets the interface visible
	 */
	public void setmainVisible() {
		setVisible(true);		
	}

}
